/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.uinfo.vegetationzones;

import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collection;

import org.apache.commons.lang.math.DoubleRange;
import org.apache.log4j.Logger;
import org.dive4elements.river.artifacts.common.I18NStrings;
import org.dive4elements.river.artifacts.common.JasperReporter;
import org.dive4elements.river.artifacts.common.MetaAndTableJRDataSource;
import org.dive4elements.river.artifacts.common.ResultRow;
import org.dive4elements.river.artifacts.sinfo.util.RiverInfo;
import org.dive4elements.river.artifacts.uinfo.commons.AbstractUInfoExporter;
import org.dive4elements.river.artifacts.uinfo.commons.UInfoResultType;

import au.com.bytecode.opencsv.CSVWriter;
import net.sf.jasperreports.engine.JRException;

/**
 * @author Domenico Nardi Tironi
 *
 */
public class VegetationZonesExporter extends AbstractUInfoExporter<VegetationZonesCalculationResult, VegetationZonesCalculationResults> {

    /** The log used in this exporter. */
    private static Logger log = Logger.getLogger(VegetationZonesExporter.class);

    private static final String JASPER_FILE = "/jasper/templates/uinfo.vegetationzones.jrxml";

    @Override
    protected Logger getLog() {
        return log;
    }

    @Override
    protected void writeCSVGlobalMetadata(final CSVWriter writer, final VegetationZonesCalculationResults results) {
        log.info("SalixLineExporter.writeCSVMeta");
        super.writeCSVGlobalMetadataDefaults(writer, results);

        final DoubleRange calcRange = results.getCalcRange();
        if (calcRange != null) {
            // "# Ort/Bereich (km): "
            writeCSVMetaEntry(writer, I18NStrings.CSV_META_RANGE, msg(I18NStrings.CSV_META_RANGE_LABEL), getKmFormatter().format(calcRange.getMinimumDouble()),
                    getKmFormatter().format(calcRange.getMaximumDouble()));
        }
    }

    /**
     * Write the header, with different headings depending on whether at a
     * gauge or at a location.
     *
     * @param river
     * @param useTkh
     */
    @Override
    protected void writeCSVHeader(final CSVWriter writer, final VegetationZonesCalculationResults results, final RiverInfo river) {
        log.info("FlowDepthExporter.writeCSVHeader");

        final Collection<String> header = new ArrayList<>(11);

        header.add(msg(UInfoResultType.vegname.getCsvHeader()));
        header.add(msg(UInfoResultType.vegdauervon.getCsvHeader()));
        header.add(msg(UInfoResultType.vegdauerbis.getCsvHeader()));

        writer.writeNext(header.toArray(new String[header.size()]));
    }

    /**
     * Format a row of a flow depth result into an array of string, both used by csv and pdf
     *
     * @param pdf
     *
     * @param useTkh
     */
    @Override
    protected String[] formatRow(final VegetationZonesCalculationResults results, final ResultRow row, final ExportMode mode) {

        final Collection<String> lines = new ArrayList<>(11);

        lines.add(row.exportValue(this.context, UInfoResultType.vegname));
        lines.add(row.exportValue(this.context, UInfoResultType.vegdauervon));
        lines.add(row.exportValue(this.context, UInfoResultType.vegdauerbis));

        return lines.toArray(new String[lines.size()]);// TEST
		
    }

    @Override
    protected void writeCSVResultMetadata(final CSVWriter writer, final VegetationZonesCalculationResults results,
            final VegetationZonesCalculationResult result) {

    }

    @Override
    protected final void addJRMetaData(final MetaAndTableJRDataSource source, final VegetationZonesCalculationResults results) {

        super.addJRMetaData(source, results);

        /* additional column headings */
        source.addMetaData("veg_name", UInfoResultType.vegname.getPdfHeader(this.context.getMeta()));
        source.addMetaData("veg_dauervon", UInfoResultType.vegdauervon.getPdfHeader(this.context.getMeta()));
        source.addMetaData("veg_dauerbis", UInfoResultType.vegdauerbis.getPdfHeader(this.context.getMeta()));
    }

    @Override
    protected void writePDF(final OutputStream out) {
        try {
            final MetaAndTableJRDataSource source = createJRData(this.data);

            final JasperReporter reporter = new JasperReporter();
            reporter.addReport(JASPER_FILE, source);
            reporter.exportPDF(out);
        }
        catch (final JRException je) {
            getLog().warn("Error generating PDF Report!", je);
        }
    }

}